var radix = function (option) {
    this.option = {
        scrollIntLink: true,
        scrollDuration: 1000,
        scrollFix: 0,
        timeFrame: 10,
        scrollEasing: 'ease-in-out',
        autoTarBlank: true,
        svgHumberger: '',
        toggleTrigger: '',
        toggleTarget: '',
        horizontalScroll: '.horizontal-scroller',
        flexFix: '.flex-fix',
        flexFixMode: false
    };
    this.baseElement = null;
    Object.assign(this.option, option);
    this.init();
};
radix.prototype.init = function () {
    var self = this;
    var host = null;
    // Smooth scroll and Auto fill target blank
    {
        // Smooth scroll
        var link = document.querySelectorAll('a');
        var scrollPos = {
            from: null,
            to: null
        };
        self.baseElement = (function () {
            var webkitFlg = navigator.userAgent.toLowerCase().match(/webkit/) ? true : false;
            return webkitFlg ? document.body : document.documentElement;
        })();
        link.forEach(function (e) {
            if (self.option.scrollIntLink) {
                e.addEventListener('click', function (v) {
                    var href = e.getAttribute('href').substr('1');
                    var target = document.getElementById(href);
                    if (href === '' || !target) {return;}
                    v.preventDefault();
                    var scrollFix = (function (fix) {
                        if (typeof fix === 'function') {
                            return fix();
                        } else {
                            return parseInt(fix, 10);
                        }
                    })(self.option.scrollFix);
                    console.log(scrollFix);
                    scrollPos.from = window.scrollY;
                    scrollPos.to = (function () {
                        var clientRect = target.getBoundingClientRect();
                        return window.scrollY + clientRect.top + scrollFix;
                    })();
                    self.move(scrollPos);
                });
            }
            // Auto fill target blank
            if (self.option.autoTarBlank) {
                var href = e.getAttribute('href');
                if (href.match(/.+\.pdf$/)) {
                    e.setAttribute('target', '_blank');
                    e.classList.add("pdf");
                } else if (href.match(/^http/)) {
                    host = location.hostname;
                    if (host === '') {
                        e.setAttribute('target', '_blank');
                        e.classList.add("extlink");
                    } else if (href.indexOf(host) < 0) {
                        e.setAttribute('target', '_blank');
                        e.classList.add("extlink");
                    }
                }
            }
        });
    }
    // SVG humberger menu insert
    if(self.option.svgHumberger !== "") {
        var svgTarget = document.querySelectorAll(self.option.svgHumberger);
        var svgHumberger = document.createElementNS('http://www.w3.org/2000/svg', 'svg');
        var svgLine1 = document.createElementNS('http://www.w3.org/2000/svg', 'line');
        var svgLine2 = document.createElementNS('http://www.w3.org/2000/svg', 'line');
        var svgLine3 = document.createElementNS('http://www.w3.org/2000/svg', 'line');
        svgHumberger.setAttribute('viewBox', '0 0 50 50');
        svgHumberger.setAttribute('xmlns', 'http://www.w3.org/2000/svg');
        svgHumberger.classList.add('rdx-svg-humberger');
        svgLine1.setAttribute('x1', 5);
        svgLine1.setAttribute('y1', 13);
        svgLine1.setAttribute('x2', 45);
        svgLine1.setAttribute('y2', 13);
        svgLine1.classList.add('bar1');
        svgLine2.setAttribute('x1', 5);
        svgLine2.setAttribute('y1', 25);
        svgLine2.setAttribute('x2', 45);
        svgLine2.setAttribute('y2', 25);
        svgLine2.classList.add('bar2');
        svgLine3.setAttribute('x1', 5);
        svgLine3.setAttribute('y1', 37);
        svgLine3.setAttribute('x2', 45);
        svgLine3.setAttribute('y2', 37);
        svgLine3.classList.add('bar3');
        svgHumberger.appendChild(svgLine1);
        svgHumberger.appendChild(svgLine2);
        svgHumberger.appendChild(svgLine3);
        svgTarget.forEach(function(e) {
            e.innerHTML = '';
            e.appendChild(svgHumberger);
        });
    }
    // Toggle menu
    if(self.option.toggleTrigger !== '' && self.option.toggleTarget !== '') {
        var toggleTrigger = document.querySelector(self.option.toggleTrigger);
        var toggleTarget = document.querySelector(self.option.toggleTarget);
        var bars = toggleTrigger.querySelectorAll('.rdx-svg-humberger line');
        var opened = false;
        toggleTrigger.addEventListener('click', function(v) {
            v.preventDefault();
            if (opened) {
                opened = false;
                bars.forEach(function(bar) {
                    bar.classList.remove('opened');
                });
                toggleTarget.classList.remove('opened');
            } else {
                opened = true;
                bars.forEach(function(bar) {
                    bar.classList.add('opened');
                });
                toggleTarget.classList.add('opened');
            }
        });
    }
    // horizontal scroller
    var horizontalScrollWrapper = document.querySelectorAll(self.option.horizontalScroll);
    if(horizontalScrollWrapper.length > 0) {
        // scroll hint
        {
            var scrollHint = document.createElement('div');
            var scrollHintIcon = document.createElement('i');
            scrollHint.classList.add('rdx-scroll-hint');
            scrollHintIcon.classList.add('fas');
            scrollHintIcon.classList.add('fa-arrows-alt-h');
            scrollHint.appendChild(scrollHintIcon); 
        } // end scroll hint
        var downEvent = function(v) {
            this.setAttribute('data-focus-on', true);
            this.setAttribute('data-already-scrolled-x', this.scrollLeft);
            this.setAttribute('data-already-scrolled-y', this.scrollTop);
            this.setAttribute('data-click-x', v.clientX);
            this.setAttribute('data-click-y', v.clientY);
        };
        var moveEvent = function(v) {
            var scrollTarget = document.querySelector('[data-focus-on="true"]');
            if(scrollTarget !== null) {
                v.preventDefault();
                scrollTarget.scrollLeft = Number(scrollTarget.getAttribute('data-already-scrolled-x')) + Number(scrollTarget.getAttribute('data-click-x')) - v.clientX;
                scrollTarget.scrollTop = Number(scrollTarget.getAttribute('data-already-scrolled-y')) + Number(scrollTarget.getAttribute('data-click-y')) - v.clientY;
            }
        };
        var upEvent = function() {
            horizontalScrollWrapper.forEach(function(e) {
                e.setAttribute('data-focus-on', false);
            });
        };
        var toggleHint = function() {
            var scrollTarget = document.querySelector('[data-focus-on="true"]');
            if(scrollTarget !== null) {
                var thisHint = scrollTarget.querySelector('.rdx-scroll-hint');
                if(thisHint !== null && scrollTarget.scrollLeft > 0) {
                    thisHint.classList.add('hide');
                }
            }
        };
        horizontalScrollWrapper.forEach(function(e) {
            e.style.overflow = 'auto';
            e.style.position = 'relative';
            e.addEventListener('mousedown', downEvent, false);
            e.addEventListener('touchstart', function() {
                e.setAttribute('data-focus-on', true);
            }, false);
            e.addEventListener('touchend', function() {
                e.setAttribute('data-focus-on', false);
            }, false);
            if(e.scrollWidth > e.clientWidth) {
                e.style.cursor = 'move';
                e.appendChild(scrollHint.cloneNode(true));
            }
        });
        document.addEventListener('mousemove', moveEvent, false);
        document.addEventListener('mousemove', toggleHint, false);
        document.addEventListener('touchmove', toggleHint, false);
        document.addEventListener('mouseup', upEvent, false);
    }
    // flex fix
    var flexFix = document.querySelectorAll(self.option.flexFix);
    if(flexFix.length > 0) {

        var flexFixMode = self.option.flexFixMode;
        flexFix.forEach(function(e) {
            var childArr = Array.from(e.children);
            var dummyChild = childArr[0].cloneNode(flexFixMode);
            dummyChild.classList.add('rdx-dummy-item');
            for(var j = 0;j < childArr.length;j++) {
                var dummyClone = dummyChild.cloneNode(flexFixMode);
                e.appendChild(dummyClone);
            } 
        }); 
    }
    // modal
    var modals = document.querySelectorAll('[data-modal-target],[data-modal-self]');
    if (modals.length > 0) {
        var modalViewport = document.createElement('div');
        var modalWrapper = document.createElement('div');
        var modalContent = document.createElement('div');
        var modalToggles = document.createElement('div');
        modalViewport.classList.add('rdx-modal-viewport');
        modalWrapper.classList.add('rdx-modal-wrapper');
        modalContent.classList.add('rdx-modal-content');
        modalToggles.classList.add('rdx-modal-toggles');
        modalViewport.appendChild(modalToggles);
        modalViewport.appendChild(modalWrapper);
        modalWrapper.appendChild(modalContent);
        var svgModal = document.createElementNS('http://www.w3.org/2000/svg', 'svg');
        var svgModalLine1 = document.createElementNS('http://www.w3.org/2000/svg', 'line');
        var svgModalLine2 = document.createElementNS('http://www.w3.org/2000/svg', 'line');
        svgModal.setAttribute('viewBox', '0 0 50 50');
        svgModal.setAttribute('xmlns', 'http://www.w3.org/2000/svg');
        svgModal.classList.add('rdx-svg-modal');
        svgModalLine1.setAttribute('x1', 15);
        svgModalLine1.setAttribute('y1', 15);
        svgModalLine1.setAttribute('x2', 35);
        svgModalLine1.setAttribute('y2', 35);
        svgModalLine1.classList.add('bar');
        svgModalLine2.setAttribute('x1', 15);
        svgModalLine2.setAttribute('y1', 35);
        svgModalLine2.setAttribute('x2', 35);
        svgModalLine2.setAttribute('y2', 15);
        svgModalLine2.classList.add('bar');
        svgModal.appendChild(svgModalLine1);
        svgModal.appendChild(svgModalLine2);
        modalToggles.appendChild(svgModal);
        var modalEnlarge = document.createElement('div');
        var modalShrink = document.createElement('div');
        modalEnlarge.classList.add('rdx-modal-enlarge');
        modalEnlarge.innerHTML = '拡大';
        modalShrink.classList.add('rdx-modal-shrink');
        modalShrink.innerHTML = '縮小';
        modalToggles.appendChild(modalEnlarge);
        modalToggles.appendChild(modalShrink);
        document.body.appendChild(modalViewport);
        modals.forEach(function(modal) {
            var targets = document.querySelectorAll(modal.getAttribute('data-modal-target'));
            targets.forEach(function(target) {
                target.classList.add('rdx-modal-source');
            });
            if(modal.getAttribute('data-modal-self')) {
                modal.classList.add('rdx-modal-source');
            }
            modal.addEventListener('click', function (v) {
                v.preventDefault();
                modalOpen(modal);
                var innerModals = modalContent.querySelectorAll('[data-modal-target],[data-modal-self]');
                innerModals.forEach(function(innerModal) {
                    innerModal.addEventListener('click', function (v) {
                        v.preventDefault();
                        modalOpen(innerModal);
                    });
                });
            });
        });
        var modalOpen = function (modal) {
            modalContent.innerHTML= '';
            var targets = document.querySelectorAll(modal.getAttribute('data-modal-target'));
            var htmlAndBody = document.querySelectorAll('html,body');
            if(modal.getAttribute('data-modal-self')) {
                var modalClone = modal.cloneNode(true);
                modalClone.removeAttribute('data-modal-self');
                modalClone.removeAttribute('data-modal-target');
                modalClone.classList.remove('rdx-modal-source');
                modalContent.appendChild(modalClone);
            } else if(modal.getAttribute('data-modal-target')) {
                targets.forEach(function(target) {
                    var modalClone = target.cloneNode(true);
                    modalClone.removeAttribute('data-modal-self');
                    modalClone.removeAttribute('data-modal-target');
                    modalClone.classList.remove('rdx-modal-source');
                    modalContent.appendChild(modalClone);
                });
            }
            modalViewport.classList.add('active');
            htmlAndBody.forEach(function(e) {
                e.style.overflowY='hidden';
            });
        };
        var modalClose = function() {
            var htmlAndBody = document.querySelectorAll('html,body');
            modalViewport.classList.remove('active');
            modalContent.innerHTML = '';
            htmlAndBody.forEach(function(e) {
                e.style.overflowY='auto';
            });
            modalContent.style.transform='scale(1)';
        };
        modalViewport.addEventListener('click', function(v) {
            var clicked = v.target;
            if(!clicked.getAttribute('data-modal-self') && !clicked.getAttribute('data-modal-target')) {
                if(clicked.closest('.rdx-modal-content') === null && clicked.closest('.rdx-modal-toggles') === null) {
                    modalClose();
                }
            }
        }, false);
        svgModal.addEventListener('click', function(){modalClose();}, false);
        var modalScale = 1;
        modalEnlarge.addEventListener('click', function(v) {
            v.preventDefault();
            if(modalScale < 1) {
                modalScale=modalScale+0.2;
                modalContent.style.transform='scale('+modalScale+')';
            } else if(modalScale < 5) {
                modalScale=modalScale+0.5;
                modalContent.style.transform='scale('+modalScale+')';
            }
        });
        modalShrink.addEventListener('click', function(v) {
            v.preventDefault();
            if(modalScale > 1) {
                modalScale=modalScale-0.5;
                modalContent.style.transform='scale('+modalScale+')';
            } else if(modalScale > 0.21) {
                modalScale=modalScale-0.2;
                modalContent.style.transform='scale('+modalScale+')';
            }
        });
    }
};
// page scroll animation
radix.prototype.move = function (scrollPos) {
    var self = this;
    var duration = self.option.scrollDuration;
    var frame = self.option.timeFrame;
    var posFrom = scrollPos.from;
    var posTo = scrollPos.to < 0 ? 0 : scrollPos.to;
    var changeVal = posTo - posFrom;
    var easing = easingFunc(self.option.scrollEasing);
    var cnt = 0;
    var timer = null;

    function moveAnimate() {
        cnt++;
        var elapsedTime = cnt * frame;
        if (elapsedTime > duration) {
            window.scrollTo(0, posTo);
            clearInterval(timer);
        }
        var pos = easing(elapsedTime, posFrom, changeVal, duration);
        window.scrollTo(0, pos);
    }
    timer = setInterval(moveAnimate, frame);
};

// easing関数取得
var easingFunc = function (name) {
    switch (name) {
        case 'linear':
            return function (t, b, c, d) {
                return c * t / d + b;
            };
        case 'ease-in':
            return function (t, b, c, d) {
                t /= d;
                return c * t * t * t + b;
            };
        case 'ease-out':
            return function (t, b, c, d) {
                t /= d;
                t--;
                return c * (t * t * t + 1) + b;
            };
        case 'ease-in-out':
            return function (t, b, c, d) {
                t /= d / 2;
                if (t < 1) return c / 2 * t * t * t + b;
                t -= 2;
                return c / 2 * (t * t * t + 2) + b;
            };
        default:
            return function (t, b, c, d) {
                return c * t / d + b;
            };
    }
};